package com.example.modacon;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.app.TabActivity;
import android.os.Bundle;
import android.graphics.Color;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.widget.TabHost.TabContentFactory;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.ArrayAdapter;
import android.widget.AdapterView;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.content.Intent;
import com.dodo.tools.CommSerial;
import com.dodo.tools.CommBluetooth;
import com.dodo.tools.CommSocket;


public class MainActivity extends TabActivity {
    private Activity  activity = this;
    private Resources res;

    private static final String PREFS = "modacon", CFG_EXT = ".dev";
    
    private static final float  SEP_SIZE  = 18F;
    private static final int    SEP_COLOR = Color.GRAY;
    
    private static final int    COMM_USB=0, COMM_BT=1, COMM_TCP=2;
    
    private ArrayAdapter<String> devsAdapter,langsAdapter, commTypeAdapter, btDevsAdapter,usbDevsAdapter;
    
    private ModbusManager manager = new ModbusManager(this);
    
    private int commType, testId;
    
    private Intent starterIntent;
    private boolean startFlag;

    @Override protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        res = getResources();
        
        starterIntent = getIntent();
        startFlag = true;

        // add default tabs
        addTab("tab1", "⌘",  R.id.tab1);
        addTab("tab2", "tt", R.id.tab2);

        // load preferences
        prefsLoad();
        
        // parse device configuration
        parseDev(((Spinner)findViewById(R.id.spDevType)).getSelectedItem().toString());
        
        // show/hide test tab
        showTestTab(false);
        ((CheckBox)findViewById(R.id.cbTest)).setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener(){
            @Override public void onCheckedChanged(CompoundButton cb, boolean state){
                manager.polling.set(!state);
                showTestTab(state);
            }
        });
        
        // channel type
        commTypeAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, new String[]{"usb","bt","tcp"});
        ((Spinner)findViewById(R.id.spCommType)).setAdapter(commTypeAdapter);
        ((Spinner)findViewById(R.id.spCommType)).setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            @Override public void onItemSelected(AdapterView<?> parent, View view, int position, long id){
                commType = position;
                ((LinearLayout)findViewById(R.id.layUsb)).setVisibility((position == COMM_USB) ? View.VISIBLE : View.GONE);
                ((LinearLayout)findViewById(R.id.layBt)).setVisibility((position == COMM_BT) ? View.VISIBLE : View.GONE);
                ((LinearLayout)findViewById(R.id.layTcp)).setVisibility((position == COMM_TCP) ? View.VISIBLE : View.GONE);
                ((LinearLayout)findViewById(R.id.layComm)).invalidate();
            }
            @Override public void onNothingSelected(AdapterView<?> parent){}
        });
        ((Spinner)findViewById(R.id.spCommType)).setSelection(commType);
        
        // USB devices
        usbDevsAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, new ArrayList<String>());
        ((Spinner)findViewById(R.id.spUsbDevs)).setAdapter(usbDevsAdapter);
        updateUsb();
        ((Spinner)findViewById(R.id.spUsbDevs)).setOnLongClickListener(new View.OnLongClickListener(){
            @Override public boolean onLongClick(View v){
                updateUsb();
                return true;
            }
        });
        
        // BT devices
        btDevsAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, new ArrayList<String>());
        ((Spinner)findViewById(R.id.spBtDevs)).setAdapter(btDevsAdapter);
        // update Bluetooth list
        updateBt();
        ((Spinner)findViewById(R.id.spBtDevs)).setOnLongClickListener(new View.OnLongClickListener(){
            @Override public boolean onLongClick(View v){
                updateBt();
                return true;
            }
        });
        
        // connect to channel
        ((Button)findViewById(R.id.btConnect)).setOnClickListener(new View.OnClickListener(){
            @Override public void onClick(View v){
                if(manager.channel == null){ // open new connection
                    (new Thread(new Runnable(){ public void run(){ if(!manager.connected()){
                        manager.slaveAddr.set(Integer.parseInt(((EditText)findViewById(R.id.edSlaveAddr)).getText().toString()));
                        manager.connecting.set(true);
                        activity.runOnUiThread(new Runnable(){ public void run(){
                            ((Button)findViewById(R.id.btConnect)).setText(res.getString(R.string.wait_sign));
                        }});
                        switch(commType){
                            case COMM_USB:
                                CommSerial usb = new CommSerial();
                                usb.open("" + ((Spinner)findViewById(R.id.spUsbDevs)).getSelectedItem());
                                if(usb.isOpen.get()){
                                    usb.configSerial(((Spinner)findViewById(R.id.spUsbBauds)).getSelectedItem().toString(), "none", "1.0");
                                    manager.connect(usb);
                                }
manager.toast("Connecting to Serial: " + usb.name + " : " + usb.isOpen.get(), true);
                                break;
                            case COMM_BT:
                                CommBluetooth bt = new CommBluetooth();
                                bt.open("" + ((Spinner)findViewById(R.id.spBtDevs)).getSelectedItem());
                                manager.connect(bt);
manager.toast("Connecting to Bluetooth: " + bt.name + " : " + bt.isOpen.get(), true);
                                break;
                            case COMM_TCP:
                                CommSocket tcp = new CommSocket();
                                tcp.open(((EditText)findViewById(R.id.edTcpAddr)).getText().toString() + ":" + ((EditText)findViewById(R.id.edTcpPort)).getText().toString());
                                manager.connect(tcp);
manager.toast("Connecting to Socket: " + tcp.name + " : " + tcp.isOpen.get(), true);
                                break;
                        }
                        manager.connecting.set(false);
                        activity.runOnUiThread(new Runnable(){ public void run(){
                            ((Button)findViewById(R.id.btConnect)).setText(res.getString( manager.connected() ? R.string.disconnect_sign : R.string.connect_sign ));
                        }});
                        // read registers
                        if(manager.connected()){
                            disableUI();
                            manager.readAll();
                        }
                    }}})).start();
                }else{
if(manager.connected()) manager.toast("Disconnecting...", true);
                    if(manager.connected()){
                        enableUI();
                        manager.disconnect();
                    }
                    activity.runOnUiThread(new Runnable(){ public void run(){
                        ((Button)findViewById(R.id.btConnect)).setText(res.getString(R.string.connect_sign));
                    }});
                }
            }
        });
        
        // test - read registers
        ((Button)findViewById(R.id.btTestRdReg)).setOnClickListener(new View.OnClickListener(){
            @Override public void onClick(View v){
                testId = manager.readRegs(
                    Integer.parseInt(((EditText)findViewById(R.id.edTestRdAddr)).getText().toString()),
                    Integer.parseInt(((EditText)findViewById(R.id.edTestRdReg)).getText().toString()),
                    Integer.parseInt(((EditText)findViewById(R.id.edTestRdRegs)).getText().toString()),
                    false
                );
                (new Timer()).schedule(new TimerTask(){ public void run(){
                    runOnUiThread(new Runnable(){ public void run(){
                        ModbusReply rep = manager.getReply(testId, true);
                        ((TextView)findViewById(R.id.txtTestRdRegs)).setText((rep == null) ? "[]" : Arrays.toString(rep.regs));
                        ((Button)findViewById(R.id.btTestRdReg)).setTextColor(((rep == null)||(rep.regs.length == 0)) ? Color.RED : Color.BLUE);
                    }});
                }}, ModbusManager.TEST_DELAY);
            }
        });
        
        // test - write register
        ((Button)findViewById(R.id.btTestWrReg)).setOnClickListener(new View.OnClickListener(){
            @Override public void onClick(View v){
                testId = manager.writeRegs(
                    Integer.parseInt(((EditText)findViewById(R.id.edTestWrAddr)).getText().toString()),
                    Integer.parseInt(((EditText)findViewById(R.id.edTestWrReg)).getText().toString()),
                    new int[]{Integer.parseInt(((EditText)findViewById(R.id.edTestWrValue)).getText().toString())},
                    false
                );
                (new Timer()).schedule(new TimerTask(){ public void run(){
                    runOnUiThread(new Runnable(){ public void run(){
((TextView)findViewById(R.id.txtTestWrRegs)).setText(manager.debugS);
//                        ModbusReply rep = manager.getReply(testId, true);
//                        ((TextView)findViewById(R.id.txtTestWrRegs)).setText(((rep == null)||(rep.regs == null)) ? "[false]" : "[true]");
//                        ((Button)findViewById(R.id.btTestWrReg)).setTextColor(((rep == null)||(rep.regs == null)) ? Color.RED : Color.BLUE);
                    }});
                }}, ModbusManager.TEST_DELAY);
            }
        });
        
        startFlag = false;
    }
    @Override protected void onDestroy(){
        manager.stop();
        super.onDestroy();
    }
    @Override protected void onResume(){
        manager.polling.set(true);
        super.onResume();
    }
    @Override protected void onPause(){
        prefsSave();
        manager.polling.set(false);
        super.onPause();
    }

    private int spDevItem;
    private void prefsLoad(){
        SharedPreferences prefs = getSharedPreferences(PREFS, 0);
        
        devsAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
        try{ 
            String[] devs = getAssets().list("");
            Arrays.sort(devs);
            devsAdapter.add("-");
            for(String dev : devs) if(dev.endsWith(CFG_EXT)) devsAdapter.add(dev.substring(0, dev.length()-CFG_EXT.length()));
        } catch(Exception e){};
        ((Spinner)findViewById(R.id.spDevType)).setAdapter(devsAdapter);
        ((Spinner)findViewById(R.id.spDevType)).setSelection(prefs.getInt("modbus.dev", 0));
        spDevItem = prefs.getInt("modbus.dev", 0);
        ((Spinner)findViewById(R.id.spDevType)).setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id){
                if(spDevItem != position){
                    spDevItem = position;
//                    prefsSave();
                    activity.recreate();
                }
            }
            public void onNothingSelected(AdapterView<?> parent){}
        });

        langsAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
        ((Spinner)findViewById(R.id.spLang)).setAdapter(langsAdapter);
        manager.lang = prefs.getInt("modbus.lang", 0);
        ((Spinner)findViewById(R.id.spLang)).setSelection(manager.lang);
        ((Spinner)findViewById(R.id.spLang)).setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id){
                if(manager.lang != position){
                    manager.lang = position;
//                    prefsSave();
                    activity.recreate();
                }
            }
            public void onNothingSelected(AdapterView<?> parent){}
        });

        ((EditText)findViewById(R.id.edSlaveAddr)).setText(prefs.getString("modbus.addr", "1"));

        commType = prefs.getInt("comm.type", 0);

        ((Spinner)findViewById(R.id.spUsbBauds)).setAdapter(new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1,
            new String[] {"1200","2400","4800","9600","14400","19200","38400","57600","115200"})
        );
        ((Spinner)findViewById(R.id.spUsbBauds)).setSelection(prefs.getInt("usb.baud",0));

        ((EditText)findViewById(R.id.edTcpAddr)).setText(prefs.getString("tcp.addr", "192.168.0.1"));
        ((EditText)findViewById(R.id.edTcpPort)).setText(prefs.getString("tcp.port", "8888"));

        ((EditText)findViewById(R.id.edTestWrAddr)).setText(prefs.getString("modbus.wr.addr", "1"));
        ((EditText)findViewById(R.id.edTestWrReg)).setText(prefs.getString("modbus.wr.reg", "0"));
        ((EditText)findViewById(R.id.edTestWrValue)).setText(prefs.getString("modbus.wr.value", "0"));

        ((EditText)findViewById(R.id.edTestRdAddr)).setText(prefs.getString("modbus.rd.addr", "1"));
        ((EditText)findViewById(R.id.edTestRdReg)).setText(prefs.getString("modbus.rd.reg", "0"));
        ((EditText)findViewById(R.id.edTestRdRegs)).setText(prefs.getString("modbus.rd.regs", "1"));
    }
    private void prefsSave(){
        SharedPreferences.Editor ed = getSharedPreferences(PREFS, 0).edit();
        
        ed.putInt("modbus.dev",         ((Spinner)findViewById(R.id.spDevType)).getSelectedItemPosition());
        ed.putInt("modbus.lang",        ((Spinner)findViewById(R.id.spLang)).getSelectedItemPosition());
        ed.putString("modbus.addr",     ((EditText)findViewById(R.id.edSlaveAddr)).getText().toString());
        
        ed.putInt("comm.type", commType);
        ed.putInt("usb.baud",           ((Spinner)findViewById(R.id.spUsbBauds)).getSelectedItemPosition());
        ed.putString("tcp.addr",        ((EditText)findViewById(R.id.edTcpAddr)).getText().toString());
        ed.putString("tcp.port",        ((EditText)findViewById(R.id.edTcpPort)).getText().toString());

        ed.putString("modbus.wr.addr",  ((EditText)findViewById(R.id.edTestWrAddr)).getText().toString());
        ed.putString("modbus.wr.reg",   ((EditText)findViewById(R.id.edTestWrReg)).getText().toString());
        ed.putString("modbus.wr.value", ((EditText)findViewById(R.id.edTestWrValue)).getText().toString());

        ed.putString("modbus.rd.addr",  ((EditText)findViewById(R.id.edTestRdAddr)).getText().toString());
        ed.putString("modbus.rd.reg",   ((EditText)findViewById(R.id.edTestRdReg)).getText().toString());
        ed.putString("modbus.rd.regs",  ((EditText)findViewById(R.id.edTestRdRegs)).getText().toString());

        ed.commit();
    }
    
    // update list of USB devices
    private void updateUsb(){
        usbDevsAdapter.clear();
        usbDevsAdapter.addAll(CommSerial.getPorts(this));
    }
    // update list of Bluetooth devices
    private void updateBt(){
        btDevsAdapter.clear();
        btDevsAdapter.addAll(CommBluetooth.pairedNames());
    }

    private int[] uiViews = {R.id.spDevType,R.id.spLang,R.id.edSlaveAddr,R.id.spCommType,R.id.spUsbDevs,R.id.spUsbBauds,R.id.spBtDevs,R.id.edTcpAddr,R.id.edTcpPort};
    private void enableUI(){
        runOnUiThread(new Runnable(){ public void run(){
            for(int id : uiViews) ((View)findViewById(id)).setEnabled(true);
        }});
    }
    private void disableUI(){
        runOnUiThread(new Runnable(){ public void run(){
            for(int id : uiViews) ((View)findViewById(id)).setEnabled(false);
        }});
    }
    
    private void addTab(String tag, String title, int contentID){
        TabHost.TabSpec tabSpec = getTabHost().newTabSpec(tag);
        tabSpec.setIndicator(title, this.getResources().getDrawable(android.R.drawable.star_on));
        tabSpec.setContent(contentID);
        getTabHost().addTab(tabSpec);
    }
    
    private int tabsNum=0;
    private HashMap<String,View> tabViews = new HashMap<String,View>();
    // parse/populate device config/tabs
    private void parseDev(String devName){
//toast("parsing: " + devName, false);
        try{
            BufferedReader br = new BufferedReader(new InputStreamReader(getAssets().open(devName + CFG_EXT),"UTF-8"));
            String line;
            LinearLayout tabLayout = null;
            while((line = br.readLine()) != null){
                // skip comment
                if((line.length() < 2) || line.startsWith("#")) continue;
                String[] words = line.replace("\r"," ").replace("\t"," ").trim().split("\\s+");
                
                // set translation languagesLinearLayout
                if((words.length == 2)&& words[0].equals("langs")){
                    langsAdapter.clear();
                    for(String lang : words[1].split(",")) langsAdapter.add(lang);
                }
                // add new tab
                else if((words.length == 2)&& words[0].equals("tab")){
                    tabsNum++;
                    String tabName = manager.tr(words[1]);
                    String tabTag = "devTab" + tabsNum;
                    TabSpec tab = getTabHost().newTabSpec(tabTag);
                    tab.setIndicator(tabName);
                    
                    tabLayout = new LinearLayout(this){{ setOrientation(LinearLayout.VERTICAL); }};
                    tab.setContent( new TabContentFactory(){
                        public View createTabContent(String tag){ return tabViews.get(tag); }
                    });
                    ScrollView tabScroll = new ScrollView(this);
                        tabScroll.addView(tabLayout);
                        tabViews.put(tabTag, tabScroll);
                    getTabHost().addTab(tab);
                    manager.regPrev = -2;
                }
                // add separator
                else if((words.length >= 1)&& words[0].equals("sep")){
                    View sep = new View(this){{
                        setPadding(5,2,5,2);
                        setLayoutParams(new LinearLayout.LayoutParams(-1,5));
                        setBackgroundColor(Color.LTGRAY);
                    }};
                    if(words.length == 1){
                        tabLayout.addView(sep);
                    }else{
                        TextView sepLabel = new TextView(this){{
                            setTextColor(SEP_COLOR);
                            setTextSize(android.util.TypedValue.COMPLEX_UNIT_SP, SEP_SIZE);
                            setPadding(0,0,0,10);
                        }};
                        sepLabel.setText(android.text.Html.fromHtml(manager.tr(words[1])));
                        tabLayout.addView(sep);
                        tabLayout.addView(sepLabel);
                    }
                    manager.regPrev = -2;
                }
                // add register
                else if((words.length >= 5)&& words[0].equals("reg")){
//                        regText.setText("-");
                    if((words.length == 5)||(words.length == 6))
                        manager.addReg(tabsNum-1, words[1], words[4], (words.length == 6) ? words[5] : "", Integer.parseInt(words[2]), Integer.parseInt(words[3]));
                    else manager.addReg(tabsNum-1, words[1], words[6], (words.length == 8) ? words[7] : "", Integer.parseInt(words[2]), Integer.parseInt(words[3]), Integer.parseInt(words[4]), Integer.parseInt(words[5]));
                    // container layout
                    tabLayout.addView( new LinearLayout(this){{
                        setOrientation(LinearLayout.HORIZONTAL);
                        // add views co container layouts
                        addView(manager.lastReg.nameText);
                        addView(manager.lastReg.editView());
                        addView(manager.lastReg.txButton);
                    }});
//                    reg.setValue(0);
                }
                // add poll
                else if((words.length >= 4)&& words[0].equals("poll")){
                    try{ manager.addPoll(Integer.parseInt(words[3]), (words[1].equals(".")) ? ModbusReg.prevAddr : Integer.parseInt(words[1]), Integer.parseInt(words[2]));
                    }catch(Exception e){}
                }
            }
        }catch(Exception e){
if(!devName.equals("-")) manager.toast("Failed to read config  " + devName + " : " + e.toString(), true);
        }
    }
    private void showTestTab(boolean on){
        getTabHost().getTabWidget().getChildAt(1).setVisibility(on ? View.VISIBLE : View.GONE);
    }
}
