package com.example.modacon;

import java.lang.Runnable;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.EditText;
import android.widget.Button;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.CheckBox;
import android.widget.SeekBar;
import android.graphics.Color;
import java.util.Timer;
import java.util.TimerTask;

// Modbus register
public class ModbusReg {
    
    public static int prevAddr = -1;
    public static int nextAddr(){ return prevAddr+1; }
    
    public String    type, name, descr;
    public int       tab, addr, min,max, digits, div, id;
    public boolean   rw; // read/write access flag
    
    public int       value = Integer.MAX_VALUE; // current int value
    public String    valueString = "";          // current edited value string
    public boolean   valueStringOk = false;     // edited string is ok ?
    
    public  ModbusManager manager;
    public  TextView nameText;
    private EditText valueText;
    private Spinner  valueSpinner;
    private ArrayAdapter<Integer> valuesAdapter;
    private CheckBox valueCheckBox;
    private Switch   valueSwitch;
    private SeekBar  valueSeekBar;
    private LinearLayout timeLayout;
    private Spinner  minuteSpinner, secondSpinner;
    public Button    txButton;
    
    // R/O register
    public ModbusReg(int _tab, String _type, String _name, String _descr, int _addr, int _digits, ModbusManager _manager){
        this(_tab, _type, _name, _descr, _addr, _digits, -1,-1, _manager);
    }
    // register
    public ModbusReg(int _tab, String _type, String _name, String _descr, int _addr, int _digits, int _min, int _max, ModbusManager _manager){
        tab = _tab;
        type = (_type.equals("n") || _type.equals("num") || _type.equals("ed") || _type.equals("edit")) ? "edit" :
            (_type.equals("sp") || _type.equals("spin") || _type.equals("spinner")) ? "spinner" :
            (_type.equals("b") || _type.equals("bool") || _type.equals("cb") || _type.equals("checkbox")) ? "checkbox" :
            (_type.equals("sw") || _type.equals("switch")) ? "switch" :
            (_type.equals("sb") || _type.equals("seek") || _type.equals("slider")) ? "seekbar" :
            (_type.equals("t") || _type.equals("tim") || _type.equals("time") || _type.equals("ms") || _type.equals("hm")) ? "time" :
            "edit";
        name = _name; descr = _descr;
        prevAddr = addr = (_addr == -1) ? nextAddr() : _addr; digits = _digits;
        div = 1; for(int i=0; i<digits; i++) div *= 10;
        min = _min; max = _max; rw = !((min == -1)&&(max == -1));
        manager = _manager;

        // name text
        nameText = new TextView(manager.activity){{
            setText(android.text.Html.fromHtml("&nbsp;" + name));
            setTextColor(rw ? Color.BLUE : Color.DKGRAY);
            setTextSize(android.util.TypedValue.COMPLEX_UNIT_SP, 20F);
            setLayoutParams(new LinearLayout.LayoutParams(-2,-1,1));
            setGravity(android.view.Gravity.CENTER_VERTICAL | android.view.Gravity.LEFT);
        }};
            
        // show description toast on long click
        if(descr.length() > 0){
            nameText.setOnLongClickListener(new View.OnLongClickListener(){
                @Override public boolean onLongClick(View v){
                    manager.toast(descr + ((max > min) ? (" (" + min + ".." + max + ")") : ""), false);
                    return true;
                }
            });
        };
            
        // write button
        txButton = new Button(manager.activity){{ setText("↘"); }};

        // EditText
        if(type.equals("edit")){
            valueText = new EditText(manager.activity){{
                setEms(6);
                setInputType(android.text.InputType.TYPE_CLASS_PHONE);
                setGravity(android.view.Gravity.RIGHT);
            }};

            if(!rw){
                // read only register?
                valueText.setKeyListener(null);
            }else{
                // entered text change listener/verifier
                valueText.addTextChangedListener(new TextWatcher(){
                    public void afterTextChanged(Editable s){
                        valueString = s.toString();
                        valueStringOk = stringToInt(valueString) != Integer.MAX_VALUE;
                        manager.activity.runOnUiThread(new Runnable(){ public void run(){
                            valueText.setTextColor(valueStringOk ? Color.BLACK : Color.RED);
                        }});
                    }
                    public void beforeTextChanged(CharSequence s, int start, int count, int after){}
                    public void onTextChanged(CharSequence s, int start, int before, int count){}
                });
                // long click
                if((min == 0)&&(max == 1)){
                    valueText.setOnLongClickListener(new View.OnLongClickListener(){
                        @Override public boolean onLongClick(View v){
                            if((valueText.getText().length() == 0) || valueText.getText().toString().equals("0")) valueText.setText("1");
                            else if(valueText.getText().toString().equals("1")) valueText.setText("0");
                            return true;
                        }
                    });
                }
            }
        }else if(type.equals("checkbox")){
            valueCheckBox = new CheckBox(manager.activity);
            valueCheckBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener(){
                @Override public void onCheckedChanged(CompoundButton arg0, boolean arg1){
                    value = arg1 ? 1 : 0;
                    valueStringOk = true;
                    valueString = "" + value;
                }
            });
        }else if(type.equals("switch")){
            valueSwitch = new Switch(manager.activity);
            valueSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener(){
                @Override public void onCheckedChanged(CompoundButton arg0, boolean arg1){
                    value = arg1 ? 1 : 0;
                    valueStringOk = true;
                    valueString = "" + value;
//manager.toast("value: " + value, false);
                }
            });
        }else if(type.equals("spinner")){
            valueSpinner = new Spinner(manager.activity);
            valueSpinner.setGravity(android.view.Gravity.RIGHT | android.view.Gravity.CENTER_VERTICAL);
//            valueSpinner.setTextAlignment(android.view.View.TEXT_ALIGNMENT_TEXT_END);
            valuesAdapter = new ArrayAdapter<Integer>(manager.activity, R.layout.reg_spinner);//android.R.layout.simple_list_item_1);
            if(max > min) for(int i=min; i<=max; i++) valuesAdapter.add(i);
            valueSpinner.setAdapter(valuesAdapter);
            valueSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
                @Override public void onItemSelected(AdapterView<?> adapterView, View view, int position, long id){
                    value = min + position;
                    valueStringOk = true;
                    valueString = "" + value;
//manager.toast("value: " + value, false);
                }
                @Override public void onNothingSelected(AdapterView<?> adapterView){
                }
            });
        }else if(type.equals("seekbar")){
            valueSeekBar = new SeekBar(manager.activity);
            valueSeekBar.setLayoutParams(new LinearLayout.LayoutParams(-2,-1,2));
//            valueSeekBar.setMin(min);
            valueSeekBar.setMax(max-min);
            valueSeekBar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener(){
                @Override public void onStopTrackingTouch(SeekBar arg0){}
                @Override public void onStartTrackingTouch(SeekBar arg0){}
                @Override public void onProgressChanged(SeekBar arg0, int progress, boolean fromUser){
                    if(fromUser) value = min + progress;
                    valueStringOk = true;
                    valueString = "" + value;
//manager.toast("value: " + value, false);
                }
            });
        }else if(type.equals("time")){
            timeLayout = new LinearLayout(manager.activity);
            timeLayout.setOrientation(LinearLayout.HORIZONTAL);

            minuteSpinner = new Spinner(manager.activity);
            minuteSpinner.setGravity(android.view.Gravity.RIGHT | android.view.Gravity.CENTER_VERTICAL);
            ArrayAdapter<Integer> minuteAdapter = new ArrayAdapter<Integer>(manager.activity, R.layout.reg_spinner);//android.R.layout.simple_list_item_1);
            for(int i=0; i<=(max / 60); i++) minuteAdapter.add(i);
            minuteSpinner.setAdapter(minuteAdapter);
            minuteSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
                @Override public void onItemSelected(AdapterView<?> adapterView, View view, int position, long id){
                    value = position * 60 + secondSpinner.getSelectedItemPosition();
                    valueStringOk = true;
                    valueString = "" + value;
//manager.toast("value: " + value, false);
                }
                @Override public void onNothingSelected(AdapterView<?> adapterView){
                }
            });

            secondSpinner = new Spinner(manager.activity);
            secondSpinner.setGravity(android.view.Gravity.RIGHT | android.view.Gravity.CENTER_VERTICAL);
            ArrayAdapter<Integer> secondAdapter = new ArrayAdapter<Integer>(manager.activity, R.layout.reg_spinner);//android.R.layout.simple_list_item_1);
            for(int i=0; i<60; i++) secondAdapter.add(i);
            secondSpinner.setAdapter(secondAdapter);
            secondSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
                @Override public void onItemSelected(AdapterView<?> adapterView, View view, int position, long id){
                    value = minuteSpinner.getSelectedItemPosition() * 60 + position;
                    valueStringOk = true;
                    valueString = "" + value;
//manager.toast("value: " + value, false);
                }
                @Override public void onNothingSelected(AdapterView<?> adapterView){
                }
            });
            
            TextView sepText = new TextView(manager.activity);
            sepText.setText(":");
            
            timeLayout.addView(minuteSpinner);
            timeLayout.addView(sepText);
            timeLayout.addView(secondSpinner);
        }
        
        if(!rw){
            txButton.setEnabled(false);
        }else{
            // short button click - write register value
            txButton.setOnClickListener(new View.OnClickListener(){
                @Override public void onClick(View v){
                    if(manager.connected() && valueStringOk){
                        value = stringToInt(valueString);
//manager.toast("addr: " + addr + " val: " + value, false);
                        id = manager.writeReg(addr, value);
                        (new Timer()).schedule(new TimerTask(){ public void run(){
                            manager.activity.runOnUiThread(new Runnable(){ public void run(){
                                txButton.setTextColor(manager.replied(id) ? Color.GREEN : Color.RED);
                            }});
                        }}, ModbusManager.TEST_DELAY);
                    }
                }
            });
            
            // long button click - read register
            txButton.setOnLongClickListener(new View.OnLongClickListener(){
                @Override public boolean onLongClick(View v){
                    id = manager.readRegs(addr, 1, true);
                    return true;
                }
            });
        }
    }
    
    // return editor widget
    public View editView(){
        return (valueText != null) ? valueText     :
           (valueCheckBox != null) ? valueCheckBox :
             (valueSwitch != null) ? valueSwitch   :
            (valueSpinner != null) ? valueSpinner  :
            (valueSeekBar != null) ? valueSeekBar  :
              (timeLayout != null) ? timeLayout    : null;
    }
    
    // set/update new value
    public void setValue(int _value){
        value = _value;
        manager.activity.runOnUiThread(new Runnable(){ public void run(){
            if(valueText != null)          valueText.setText(valueToString());
            else if(valueCheckBox != null) valueCheckBox.setChecked(value != 0);
            else if(valueSwitch   != null) valueSwitch.setChecked(value != 0);
            else if(valueSpinner  != null) valueSpinner.setSelection(((value >= min)&&(value <= max)) ? (value-min) : valueSpinner.getSelectedItemPosition());
            else if(valueSeekBar  != null) valueSeekBar.setProgress(((value >= min)&&(value <= max)) ? (value-min) : 0);
            else if(timeLayout    != null){
                minuteSpinner.setSelection(value / 60);
                secondSpinner.setSelection(value % 60);
            }
        }});
    }
    // convert int value to string/double
    public String valueToString(){
        int v = value;
        String t = "" + v;
        if(!rw && (v == -32768)) t = "-";
        else if(digits > 0){
            t = (v / div) + ".";
            String rem = "" + (v % div);
            while(rem.length() < digits) rem = "0" + rem;
            t += rem;
        }
        return t;
    }
    // try to parse string value
    public int stringToInt(String s){
//        if((min == -1)&&(max == -1)) return true;
        try{
            int vi = -32768;
            if(s.startsWith("0x") || s.startsWith("0X")){
                vi = Integer.parseInt(s.substring(2), 16);
            }else{
                double vd = Double.parseDouble(s);
                vi = (int)Math.floor(vd * div);
            }
            if((vi < min)||(vi > max)) return Integer.MAX_VALUE;
            return vi;
        }catch(Exception e){}
        return Integer.MAX_VALUE;
    }
}
