package com.example.modacon;

public class ModbusRtu {
    public static int debugN;
    public static String debugS;
    
    public static final int CRC16_INIT = 0xFFFF;
    public static final int CRC16_POLY = 0xA001;

    // calculate RTU CRC16
    public static int crc16(byte[] data, int start, int end){
        int crc = CRC16_INIT;
        if(end <= 0) end = data.length-1;
        for(int i=start; i<=end; i++){
            crc ^= data[i] & 0xFF;
            for(int b=8; b!=0; b--){
                int f = crc & 1;
                crc >>= 1;
                if(f != 0) crc ^= CRC16_POLY;
            }
        }
        return crc;
    }
    
    // create 03 request packet
    public static byte[] create03(int addr, int reg, int n){
        byte packet[] = new byte[8];
        packet[0] = (byte)(addr & 0xFF);
        packet[1] = 3;
        packet[2] = (byte)((reg >> 8) & 0xFF);
        packet[3] = (byte)(reg & 0xFF);
        packet[4] = (byte)((n >> 8) & 0xFF);
        packet[5] = (byte)(n & 0xFF);
        int crc = crc16(packet, 0,5);
        packet[6] = (byte)(crc & 0xFF);
        packet[7] = (byte)((crc >> 8) & 0xFF);
        return packet;
    }
    // check/parse 03 response packet
    public static int[] parse03(int addr, int reg, int n, byte packet[]){
        if(packet.length >= (5 + n*2)){
            // ignore echo/garbage
            int start = packet.length - (5 + n*2);
            if(((packet[start] & 0xFF) == addr)&&((packet[start+1] & 0xFF) == 3)&&((packet[start+2] & 0xFF) == (n*2))){
                int crc = crc16(packet, start,packet.length-3);
                if(((packet[packet.length-2] & 0xFF) == (crc & 0xFF))&&((packet[packet.length-1] & 0xFF) == (crc >> 8))){
                    int regs[] = new int[n];
                    for(int i=0; i<n; i++)
                        regs[i] = (short)(((packet[start + 3 + i*2] & 0xFF) << 8) | (packet[start + 4 + i*2] & 0xFF));
                    return regs;
                }
            }
        }
        return new int[0];
    }

    // create 16 request packet
    public static byte[] create16(int addr, int reg, int regs[]){
        byte packet[] = new byte[9 + regs.length * 2];
        packet[0] = (byte)(addr & 0xFF);
        packet[1] = 16;
        packet[2] = (byte)((reg >> 8) & 0xFF);
        packet[3] = (byte)(reg & 0xFF);
        packet[4] = (byte)((regs.length >> 8) & 0xFF);
        packet[5] = (byte)(regs.length & 0xFF);
        packet[6] = (byte)(regs.length * 2 & 0xFF);
        for(int i=0; i<regs.length; i++){
            packet[7 + i*2] = (byte)((regs[i] >> 8) & 0xFF);
            packet[8 + i*2] = (byte)(regs[i] & 0xFF);
        }
        int crc = crc16(packet, 0, 6 + regs.length*2);
        packet[packet.length-2] = (byte)(crc & 0xFF);
        packet[packet.length-1] = (byte)((crc >> 8) & 0xFF);
        return packet;
    }
    // check/parse 16 response packet
    public static boolean parse16(int addr, int reg, int n, byte packet[]){
        if(packet.length >= 8){
            // ignore echo/garbage
            int start = packet.length-8;
            if(((packet[start] & 0xFF) == addr)&&((packet[start+1] & 0xFF) == 16)&&
                ((packet[start+2] & 0xFF) == (reg >> 8))&&((packet[start+3] & 0xFF) == (reg & 0xFF))&&
                ((packet[start+4] & 0xFF) == (n >> 8))&&((packet[start+5] & 0xFF) == (n & 0xFF))){
                int crc = crc16(packet, start,start+5);
                if(((packet[start+6] & 0xFF) == (crc & 0xFF))&&((packet[start+7] & 0xFF) == (crc >> 8)))
                    return true;
            }
        }
        return false;
    }
}
